;;; gdscript-keywords.el --- Cached keywords for the GDScript language  -*- lexical-binding: t; -*-

;; Copyright (C) 2020-2026 GDQuest and contributors

;; Author: Nathan Lovato <nathan@gdquest.com>
;; Maintainer: Jen-Chieh Shen <jcs090218@gmail.com>

;; This file is not part of GNU Emacs.

;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <https://www.gnu.org/licenses/>.

;;; Commentary:
;;
;; Cached keywords for the GDScript language, used for syntax
;; highlighting and auto-completion.
;;

;;; Code:

(defconst gdscript-annotations
  '("abstract"
    "export"
    "export_category"
    "export_color_no_alpha"
    "export_custom"
    "export_dir"
    "export_enum"
    "export_exp_easing"
    "export_file"
    "export_file_path"
    "export_flags"
    "export_flags_2d_navigation"
    "export_flags_2d_physics"
    "export_flags_2d_render"
    "export_flags_3d_navigation"
    "export_flags_3d_physics"
    "export_flags_3d_render"
    "export_flags_avoidance"
    "export_global_dir"
    "export_global_file"
    "export_group"
    "export_multiline"
    "export_node_path"
    "export_placeholder"
    "export_range"
    "export_storage"
    "export_subgroup"
    "export_tool_button"
    "icon"
    "onready"
    "rpc"
    "static_unload"
    "tool"
    "warning_ignore"
    "warning_ignore_restore"
    "warning_ignore_start"))

(defconst gdscript-keywords
  '("and"
    "as"
    "assert"
    "await"
    "break"
    "breakpoint"
    "case"
    "class"
    "class_name"
    "const"
    "continue"
    "do"
    "elif"
    "else"
    "enum"
    "extends"
    "false"
    "for"
    "func"
    "if"
    "in"
    "is"
    "master"
    "match"
    "not"
    "or"
    "pass"
    "preload"
    "puppet"
    "remote"
    "remotesync"
    "return"
    "self"
    "setget"
    "signal"
    "super"
    "slave"
    "static"
    "switch"
    "sync"
    "true"
    "var"
    "while"
    "yield"))

(defconst gdscript-built-in-constants
  '("INF"
    "NAN"
    "PI"
    "TAU"))

;; Contains types highlighted by text_editor/theme/highlighting/base_type_color
;; These are typically non-Object, Variant types
(defconst gdscript-built-in-types
  '("AABB"
    "Array"
    "Basis"
    "bool"
    "Callable"
    "Color"
    "Dictionary"
    "float"
    "int"
    "NodePath"
    "Nil"
    "null"
    "PackedByteArray"
    "PackedColorArray"
    "PackedFloat32Array"
    "PackedFloat64Array"
    "PackedInt32Array"
    "PackedInt64Array"
    "PackedStringArray"
    "PackedVector2Array"
    "PackedVector3Array"
    "PackedVector4Array"
    "Plane"
    "Projection"
    "Quaternion"
    "Rect2"
    "Rect2i"
    "RID"
    "Signal"
    "String"
    "StringName"
    "Transform2D"
    "Transform3D"
    "Vector2"
    "Vector2i"
    "Vector3"
    "Vector3i"
    "Vector4"
    "Vector4i"
    "void"
))

(defconst gdscript-built-in-functions
  '("Color8"
    "ColorN"
    "abs"
    "absf"
    "acos"
    "acosh"
    "angle_difference"
    "asin"
    "asinh"
    "atan"
    "atan2"
    "atanh"
    "bezier_derivative"
    "bezier_interpolate"
    "bytes2var"
    "bytes_to_var"
    "bytes_to_var_with_objects"
    "cartesian2polar"
    "ceil"
    "ceilf"
    "ceili"
    "char"
    "clamp"
    "convert"
    "cos"
    "cosh"
    "cubic_interpolate"
    "cubic_interpolate_angle"
    "cubic_interpolate_angle_in_time"
    "cubic_interpolate_in_time"
    "db2linear"
    "db_to_linear"
    "decimals"
    "dectime"
    "deg2rad"
    "deg_to_rad"
    "dict2inst"
    "dict_to_inst"
    "ease"
    "error_string"
    "exp"
    "floor"
    "floorf"
    "floori"
    "fmod"
    "fposmod"
    "funcref"
    "get_stack"
    "hash"
    "inst2dict"
    "inst_to_dict"
    "instance_from_id"
    "inverse_lerp"
    "is_equal_approx"
    "is_finite"
    "is_inf"
    "is_instance_id_valid"
    "is_instance_valid"
    "is_nan"
    "is_same"
    "is_zero_approx"
    "len"
    "lerp"
    "lerp_angle"
    "lerpf"
    "linear2db"
    "linear_to_db"
    "load"
    "log"
    "max"
    "maxf"
    "maxi"
    "min"
    "minf"
    "mini"
    "move_toward"
    "nearest_po2"
    "ord"
    "parse_json"
    "pingpong"
    "polar2cartesian"
    "posmod"
    "pow"
    "preload"
    "print"
    "print_debug"
    "print_rich"
    "print_stack"
    "print_verbose"
    "printerr"
    "printraw"
    "prints"
    "printt"
    "push_error"
    "push_warning"
    "rad2deg"
    "rad_to_deg"
    "rand_from_seed"
    "rand_range"
    "rand_seed"
    "randf"
    "randf_range"
    "randfn"
    "randi"
    "randi_range"
    "randomize"
    "range"
    "range_lerp"
    "remap"
    "rid_allocate_id"
    "rid_from_int64"
    "round"
    "roundf"
    "roundi"
    "seed"
    "sign"
    "signf"
    "signi"
    "sin"
    "sinh"
    "smoothstep"
    "sqrt"
    "step_decimals"
    "stepify"
    "str"
    "str2var"
    "str_to_var"
    "tan"
    "tanh"
    "to_json"
    "type_convert"
    "type_exists"
    "type_string"
    "typeof"
    "validate_json"
    "var2bytes"
    "var_to_bytes"
    "var_to_bytes_with_objects"
    "var2str"
    "var_to_str"
    "weakref"
    "wrap"
    "wrapf"
    "wrapi"))

;; Contains all engine classes and node types, including vectors, transforms, etc.
(defconst gdscript-built-in-classes
  '("ARVRAnchor"
    "ARVRCamera"
    "ARVRController"
    "ARVRInterface"
    "ARVROrigin"
    "ARVRPositionalTracker"
    "ARVRServer"
    "AStar"
    "AStar2D"
    "AcceptDialog"
    "AnimatedSprite"
    "AnimatedSprite3D"
    "AnimatedTexture"
    "Animation"
    "AnimationNode"
    "AnimationNodeAdd2"
    "AnimationNodeAdd3"
    "AnimationNodeAnimation"
    "AnimationNodeBlend2"
    "AnimationNodeBlend3"
    "AnimationNodeBlendSpace1D"
    "AnimationNodeBlendSpace2D"
    "AnimationNodeBlendTree"
    "AnimationNodeOneShot"
    "AnimationNodeOutput"
    "AnimationNodeStateMachine"
    "AnimationNodeStateMachinePlayback"
    "AnimationNodeStateMachineTransition"
    "AnimationNodeTimeScale"
    "AnimationNodeTimeSeek"
    "AnimationNodeTransition"
    "AnimationPlayer"
    "AnimationRootNode"
    "AnimationTrackEditPlugin"
    "AnimationTree"
    "AnimationTreePlayer"
    "Area"
    "Area2D"
    "ArrayMesh"
    "AtlasTexture"
    "AudioBusLayout"
    "AudioEffect"
    "AudioEffectAmplify"
    "AudioEffectBandLimitFilter"
    "AudioEffectBandPassFilter"
    "AudioEffectChorus"
    "AudioEffectCompressor"
    "AudioEffectDelay"
    "AudioEffectDistortion"
    "AudioEffectEQ"
    "AudioEffectEQ10"
    "AudioEffectEQ21"
    "AudioEffectEQ6"
    "AudioEffectFilter"
    "AudioEffectHighPassFilter"
    "AudioEffectHighShelfFilter"
    "AudioEffectInstance"
    "AudioEffectLimiter"
    "AudioEffectLowPassFilter"
    "AudioEffectLowShelfFilter"
    "AudioEffectNotchFilter"
    "AudioEffectPanner"
    "AudioEffectPhaser"
    "AudioEffectPitchShift"
    "AudioEffectRecord"
    "AudioEffectReverb"
    "AudioEffectSpectrumAnalyzer"
    "AudioEffectSpectrumAnalyzerInstance"
    "AudioEffectStereoEnhance"
    "AudioServer"
    "AudioStream"
    "AudioStreamGenerator"
    "AudioStreamGeneratorPlayback"
    "AudioStreamMicrophone"
    "AudioStreamPlayback"
    "AudioStreamPlaybackResampled"
    "AudioStreamPlayer"
    "AudioStreamPlayer2D"
    "AudioStreamPlayer3D"
    "AudioStreamRandomPitch"
    "AudioStreamSample"
    "BackBufferCopy"
    "BakedLightmap"
    "BakedLightmapData"
    "BaseButton"
    "BitMap"
    "BitmapFont"
    "Bone2D"
    "BoneAttachment"
    "BoxContainer"
    "BoxShape"
    "Button"
    "ButtonGroup"
    "CPUParticles"
    "CPUParticles2D"
    "Camera"
    "Camera2D"
    "CameraFeed"
    "CameraServer"
    "CameraTexture"
    "CanvasItem"
    "CanvasItemMaterial"
    "CanvasLayer"
    "CanvasModulate"
    "CapsuleMesh"
    "CapsuleShape"
    "CapsuleShape2D"
    "CenterContainer"
    "CharFXTransform"
    "CheckBox"
    "CheckButton"
    "CircleShape2D"
    "ClassDB"
    "ClippedCamera"
    "CollisionObject"
    "CollisionObject2D"
    "CollisionPolygon"
    "CollisionPolygon2D"
    "CollisionShape"
    "CollisionShape2D"
    "ColorPicker"
    "ColorPickerButton"
    "ColorRect"
    "ConcavePolygonShape"
    "ConcavePolygonShape2D"
    "ConeTwistJoint"
    "ConfigFile"
    "ConfirmationDialog"
    "Container"
    "Control"
    "ConvexPolygonShape"
    "ConvexPolygonShape2D"
    "Crypto"
    "CryptoKey"
    "CubeMap"
    "CubeMesh"
    "Curve"
    "Curve2D"
    "Curve3D"
    "CurveTexture"
    "CylinderMesh"
    "CylinderShape"
    "DampedSpringJoint2D"
    "DirectionalLight"
    "Directory"
    "DynamicFont"
    "DynamicFontData"
    "EditorExportPlugin"
    "EditorFeatureProfile"
    "EditorFileDialog"
    "EditorFileSystem"
    "EditorFileSystemDirectory"
    "EditorImportPlugin"
    "EditorInspector"
    "EditorInspectorPlugin"
    "EditorInterface"
    "EditorNavigationMeshGenerator"
    "EditorPlugin"
    "EditorProperty"
    "EditorResourceConversionPlugin"
    "EditorResourcePreview"
    "EditorResourcePreviewGenerator"
    "EditorSceneImporter"
    "EditorSceneImporterAssimp"
    "EditorScenePostImport"
    "EditorScript"
    "EditorSelection"
    "EditorSettings"
    "EditorSpatialGizmo"
    "EditorSpatialGizmoPlugin"
    "EditorSpinSlider"
    "EditorVCSInterface"
    "EncodedObjectAsID"
    "Engine"
    "Environment"
    "Expression"
    "File"
    "FileDialog"
    "Font"
    "FuncRef"
    "GIProbe"
    "GIProbeData"
    "Generic6DOFJoint"
    "Geometry"
    "GeometryInstance"
    "Gradient"
    "GradientTexture"
    "GraphEdit"
    "GraphNode"
    "GridContainer"
    "GrooveJoint2D"
    "HBoxContainer"
    "HScrollBar"
    "HSeparator"
    "HSlider"
    "HSplitContainer"
    "HTTPClient"
    "HTTPRequest"
    "HashingContext"
    "HeightMapShape"
    "HingeJoint"
    "IP"
    "IP_Unix"
    "Image"
    "ImageTexture"
    "ImmediateGeometry"
    "Input"
    "InputDefault"
    "InputEvent"
    "InputEventAction"
    "InputEventGesture"
    "InputEventJoypadButton"
    "InputEventJoypadMotion"
    "InputEventKey"
    "InputEventMIDI"
    "InputEventMagnifyGesture"
    "InputEventMouse"
    "InputEventMouseButton"
    "InputEventMouseMotion"
    "InputEventPanGesture"
    "InputEventScreenDrag"
    "InputEventScreenTouch"
    "InputEventWithModifiers"
    "InputMap"
    "InstancePlaceholder"
    "InterpolatedCamera"
    "ItemList"
    "JSON"
    "JSONParseResult"
    "JSONRPC"
    "JavaScript"
    "Joint"
    "Joint2D"
    "KinematicBody"
    "KinematicBody2D"
    "KinematicCollision"
    "KinematicCollision2D"
    "Label"
    "LargeTexture"
    "Light"
    "Light2D"
    "LightOccluder2D"
    "Line2D"
    "LineEdit"
    "LineShape2D"
    "LinkButton"
    "Listener"
    "MainLoop"
    "MarginContainer"
    "Marshalls"
    "Material"
    "MenuButton"
    "Mesh"
    "MeshDataTool"
    "MeshInstance"
    "MeshInstance2D"
    "MeshLibrary"
    "MeshTexture"
    "MultiMesh"
    "MultiMeshInstance"
    "MultiMeshInstance2D"
    "MultiplayerAPI"
    "Mutex"
    "Navigation"
    "Navigation2D"
    "NavigationMesh"
    "NavigationMeshInstance"
    "NavigationPolygon"
    "NavigationPolygonInstance"
    "NetworkedMultiplayerPeer"
    "NinePatchRect"
    "Node"
    "Node2D"
    "OS"
    "Object"
    "OccluderPolygon2D"
    "OmniLight"
    "OptionButton"
    "PCKPacker"
    "PHashTranslation"
    "PackedDataContainer"
    "PackedDataContainerRef"
    "PackedScene"
    "PacketPeer"
    "PacketPeerStream"
    "PacketPeerUDP"
    "Panel"
    "PanelContainer"
    "PanoramaSky"
    "ParallaxBackground"
    "ParallaxLayer"
    "Particles"
    "Particles2D"
    "ParticlesMaterial"
    "Path"
    "Path2D"
    "PathFollow"
    "PathFollow2D"
    "Performance"
    "PhysicalBone"
    "Physics2DDirectBodyState"
    "Physics2DDirectBodyStateSW"
    "Physics2DDirectSpaceState"
    "Physics2DServer"
    "Physics2DServerSW"
    "Physics2DShapeQueryParameters"
    "Physics2DShapeQueryResult"
    "Physics2DTestMotionResult"
    "PhysicsBody"
    "PhysicsBody2D"
    "PhysicsDirectBodyState"
    "PhysicsDirectSpaceState"
    "PhysicsMaterial"
    "PhysicsServer"
    "PhysicsShapeQueryParameters"
    "PhysicsShapeQueryResult"
    "PinJoint"
    "PinJoint2D"
    "PlaneMesh"
    "PlaneShape"
    "PointMesh"
    "Polygon2D"
    "PolygonPathFinder"
    "PoolByteArray"
    "PoolColorArray"
    "PoolIntArray"
    "PoolRealArray"
    "PoolStringArray"
    "PoolVector2Array"
    "PoolVector3Array"
    "Popup"
    "PopupDialog"
    "PopupMenu"
    "PopupPanel"
    "Position2D"
    "Position3D"
    "PrimitiveMesh"
    "PrismMesh"
    "ProceduralSky"
    "ProgressBar"
    "ProjectSettings"
    "ProximityGroup"
    "ProxyTexture"
    "QuadMesh"
    "Quat"
    "RID"
    "RandomNumberGenerator"
    "Range"
    "RayCast"
    "RayCast2D"
    "RayShape"
    "RayShape2D"
    "RectangleShape2D"
    "Reference"
    "ReferenceRect"
    "ReflectionProbe"
    "RemoteTransform"
    "RemoteTransform2D"
    "Resource"
    "ResourceFormatLoader"
    "ResourceFormatLoaderCrypto"
    "ResourceFormatSaver"
    "ResourceFormatSaverCrypto"
    "ResourceImporter"
    "ResourceInteractiveLoader"
    "ResourceLoader"
    "ResourcePreloader"
    "ResourceSaver"
    "RichTextEffect"
    "RichTextLabel"
    "RigidBody"
    "RigidBody2D"
    "RootMotionView"
    "SceneState"
    "SceneTree"
    "SceneTreeTimer"
    "Script"
    "ScriptCreateDialog"
    "ScriptEditor"
    "ScrollBar"
    "ScrollContainer"
    "SegmentShape2D"
    "Semaphore"
    "Separator"
    "Shader"
    "ShaderMaterial"
    "Shape"
    "Shape2D"
    "ShortCut"
    "Skeleton"
    "Skeleton2D"
    "SkeletonIK"
    "Skin"
    "SkinReference"
    "Sky"
    "Slider"
    "SliderJoint"
    "SoftBody"
    "Spatial"
    "SpatialGizmo"
    "SpatialMaterial"
    "SpatialVelocityTracker"
    "SphereMesh"
    "SphereShape"
    "SpinBox"
    "SplitContainer"
    "SpotLight"
    "SpringArm"
    "Sprite"
    "Sprite3D"
    "SpriteBase3D"
    "SpriteFrames"
    "StaticBody"
    "StaticBody2D"
    "StreamPeer"
    "StreamPeerBuffer"
    "StreamPeerSSL"
    "StreamPeerTCP"
    "StreamTexture"
    "StyleBox"
    "StyleBoxEmpty"
    "StyleBoxFlat"
    "StyleBoxLine"
    "StyleBoxTexture"
    "SurfaceTool"
    "TCP_Server"
    "TabContainer"
    "Tabs"
    "TextEdit"
    "TextFile"
    "Texture"
    "Texture3D"
    "TextureArray"
    "TextureButton"
    "TextureLayered"
    "TextureProgress"
    "TextureRect"
    "Theme"
    "Thread"
    "TileMap"
    "TileSet"
    "Timer"
    "ToolButton"
    "TouchScreenButton"
    "Transform"
    "Translation"
    "TranslationServer"
    "Tree"
    "TreeItem"
    "TriangleMesh"
    "Tween"
    "UndoRedo"
    "VBoxContainer"
    "VScrollBar"
    "VSeparator"
    "VSlider"
    "VSplitContainer"
    "Variant"
    "VehicleBody"
    "VehicleWheel"
    "VideoPlayer"
    "VideoStream"
    "Viewport"
    "ViewportContainer"
    "ViewportTexture"
    "VisibilityEnabler"
    "VisibilityEnabler2D"
    "VisibilityNotifier"
    "VisibilityNotifier2D"
    "VisualInstance"
    "VisualServer"
    "VisualShader"
    "VisualShaderNode"
    "VisualShaderNodeBooleanConstant"
    "VisualShaderNodeBooleanUniform"
    "VisualShaderNodeColorConstant"
    "VisualShaderNodeColorFunc"
    "VisualShaderNodeColorOp"
    "VisualShaderNodeColorUniform"
    "VisualShaderNodeCompare"
    "VisualShaderNodeCubeMap"
    "VisualShaderNodeCubeMapUniform"
    "VisualShaderNodeCustom"
    "VisualShaderNodeDeterminant"
    "VisualShaderNodeDotProduct"
    "VisualShaderNodeExpression"
    "VisualShaderNodeFaceForward"
    "VisualShaderNodeFresnel"
    "VisualShaderNodeGlobalExpression"
    "VisualShaderNodeGroupBase"
    "VisualShaderNodeIf"
    "VisualShaderNodeInput"
    "VisualShaderNodeIs"
    "VisualShaderNodeOuterProduct"
    "VisualShaderNodeOutput"
    "VisualShaderNodeScalarClamp"
    "VisualShaderNodeScalarConstant"
    "VisualShaderNodeScalarDerivativeFunc"
    "VisualShaderNodeScalarFunc"
    "VisualShaderNodeScalarInterp"
    "VisualShaderNodeScalarOp"
    "VisualShaderNodeScalarSmoothStep"
    "VisualShaderNodeScalarSwitch"
    "VisualShaderNodeScalarUniform"
    "VisualShaderNodeSwitch"
    "VisualShaderNodeTexture"
    "VisualShaderNodeTextureUniform"
    "VisualShaderNodeTextureUniformTriplanar"
    "VisualShaderNodeTransformCompose"
    "VisualShaderNodeTransformConstant"
    "VisualShaderNodeTransformDecompose"
    "VisualShaderNodeTransformFunc"
    "VisualShaderNodeTransformMult"
    "VisualShaderNodeTransformUniform"
    "VisualShaderNodeTransformVecMult"
    "VisualShaderNodeUniform"
    "VisualShaderNodeVec3Constant"
    "VisualShaderNodeVec3Uniform"
    "VisualShaderNodeVectorClamp"
    "VisualShaderNodeVectorCompose"
    "VisualShaderNodeVectorDecompose"
    "VisualShaderNodeVectorDerivativeFunc"
    "VisualShaderNodeVectorDistance"
    "VisualShaderNodeVectorFunc"
    "VisualShaderNodeVectorInterp"
    "VisualShaderNodeVectorLen"
    "VisualShaderNodeVectorOp"
    "VisualShaderNodeVectorRefract"
    "VisualShaderNodeVectorScalarMix"
    "VisualShaderNodeVectorScalarSmoothStep"
    "VisualShaderNodeVectorScalarStep"
    "VisualShaderNodeVectorSmoothStep"
    "WeakRef"
    "WindowDialog"
    "World"
    "World2D"
    "WorldEnvironment"
    "X509Certificate"
    "XMLParser"
    "YSort"
    "AESContext"
    "AStar3D"
    "AStarGrid2D"
    "AimModifier3D"
    "AnimatableBody2D"
    "AnimatableBody3D"
    "AnimatedSprite2D"
    "AnimationLibrary"
    "AnimationMixer"
    "AnimationNodeExtension"
    "AnimationNodeSub2"
    "AnimationNodeSync"
    "Area3D"
    "ArrayOccluder3D"
    "AspectRatioContainer"
    "AudioEffectCapture"
    "AudioEffectHardLimiter"
    "AudioListener2D"
    "AudioListener3D"
    "AudioSample"
    "AudioSamplePlayback"
    "AudioStreamInteractive"
    "AudioStreamMP3"
    "AudioStreamOggVorbis"
    "AudioStreamPlaybackInteractive"
    "AudioStreamPlaybackOggVorbis"
    "AudioStreamPlaybackPlaylist"
    "AudioStreamPlaybackPolyphonic"
    "AudioStreamPlaybackSynchronized"
    "AudioStreamPlaylist"
    "AudioStreamPolyphonic"
    "AudioStreamRandomizer"
    "AudioStreamSynchronized"
    "AudioStreamWAV"
    "BaseMaterial3D"
    "BoneAttachment3D"
    "BoneConstraint3D"
    "BoneMap"
    "BoneTwistDisperser3D"
    "BoxMesh"
    "BoxOccluder3D"
    "BoxShape3D"
    "CCDIK3D"
    "CPUParticles3D"
    "CSGBox3D"
    "CSGCombiner3D"
    "CSGCylinder3D"
    "CSGMesh3D"
    "CSGPolygon3D"
    "CSGPrimitive3D"
    "CSGShape3D"
    "CSGSphere3D"
    "CSGTorus3D"
    "CallbackTweener"
    "Camera3D"
    "CameraAttributes"
    "CameraAttributesPhysical"
    "CameraAttributesPractical"
    "CanvasGroup"
    "CanvasTexture"
    "CapsuleShape3D"
    "ChainIK3D"
    "CharacterBody2D"
    "CharacterBody3D"
    "CodeEdit"
    "CodeHighlighter"
    "CollisionObject3D"
    "CollisionPolygon3D"
    "CollisionShape3D"
    "ColorPalette"
    "Compositor"
    "CompositorEffect"
    "CompressedCubemap"
    "CompressedCubemapArray"
    "CompressedTexture2D"
    "CompressedTexture2DArray"
    "CompressedTexture3D"
    "CompressedTextureLayered"
    "ConcavePolygonShape3D"
    "ConeTwistJoint3D"
    "ConvertTransformModifier3D"
    "ConvexPolygonShape3D"
    "CopyTransformModifier3D"
    "Cubemap"
    "CubemapArray"
    "CurveXYZTexture"
    "CylinderShape3D"
    "DPITexture"
    "DTLSServer"
    "Decal"
    "DirAccess"
    "DirectionalLight2D"
    "DirectionalLight3D"
    "DisplayServer"
    "ENetConnection"
    "ENetMultiplayerPeer"
    "ENetPacketPeer"
    "EditorCommandPalette"
    "EditorContextMenuPlugin"
    "EditorDebuggerPlugin"
    "EditorDebuggerSession"
    "EditorDock"
    "EditorExportPlatform"
    "EditorExportPlatformAndroid"
    "EditorExportPlatformAppleEmbedded"
    "EditorExportPlatformExtension"
    "EditorExportPlatformIOS"
    "EditorExportPlatformLinuxBSD"
    "EditorExportPlatformMacOS"
    "EditorExportPlatformPC"
    "EditorExportPlatformVisionOS"
    "EditorExportPlatformWeb"
    "EditorExportPlatformWindows"
    "EditorExportPreset"
    "EditorFileSystemImportFormatSupportQuery"
    "EditorNode3DGizmo"
    "EditorNode3DGizmoPlugin"
    "EditorPaths"
    "EditorResourcePicker"
    "EditorResourceTooltipPlugin"
    "EditorSceneFormatImporter"
    "EditorSceneFormatImporterBlend"
    "EditorSceneFormatImporterFBX2GLTF"
    "EditorSceneFormatImporterGLTF"
    "EditorSceneFormatImporterUFBX"
    "EditorScenePostImportPlugin"
    "EditorScriptPicker"
    "EditorSyntaxHighlighter"
    "EditorToaster"
    "EditorTranslationParserPlugin"
    "EditorUndoRedoManager"
    "EngineDebugger"
    "EngineProfiler"
    "ExternalTexture"
    "FABRIK3D"
    "FBXDocument"
    "FBXState"
    "FastNoiseLite"
    "FileAccess"
    "FileSystemDock"
    "FlowContainer"
    "FogMaterial"
    "FogVolume"
    "FoldableContainer"
    "FoldableGroup"
    "FontFile"
    "FontVariation"
    "FramebufferCacheRD"
    "GDExtension"
    "GDExtensionManager"
    "GDScript"
    "GDScriptEditorTranslationParserPlugin"
    "GDScriptNativeClass"
    "GDScriptSyntaxHighlighter"
    "GLTFAccessor"
    "GLTFAnimation"
    "GLTFBufferView"
    "GLTFCamera"
    "GLTFDocument"
    "GLTFDocumentExtension"
    "GLTFDocumentExtensionConvertImporterMesh"
    "GLTFDocumentExtensionPhysics"
    "GLTFDocumentExtensionTextureKTX"
    "GLTFDocumentExtensionTextureWebP"
    "GLTFLight"
    "GLTFMesh"
    "GLTFNode"
    "GLTFObjectModelProperty"
    "GLTFPhysicsBody"
    "GLTFPhysicsShape"
    "GLTFSkeleton"
    "GLTFSkin"
    "GLTFSpecGloss"
    "GLTFState"
    "GLTFTexture"
    "GLTFTextureSampler"
    "GPUParticles2D"
    "GPUParticles3D"
    "GPUParticlesAttractor3D"
    "GPUParticlesAttractorBox3D"
    "GPUParticlesAttractorSphere3D"
    "GPUParticlesAttractorVectorField3D"
    "GPUParticlesCollision3D"
    "GPUParticlesCollisionBox3D"
    "GPUParticlesCollisionHeightField3D"
    "GPUParticlesCollisionSDF3D"
    "GPUParticlesCollisionSphere3D"
    "Generic6DOFJoint3D"
    "Geometry2D"
    "Geometry3D"
    "GeometryInstance3D"
    "GodotInstance"
    "GodotNavigationServer2D"
    "GodotPhysicsServer2D"
    "GodotPhysicsServer3D"
    "GradientTexture1D"
    "GradientTexture2D"
    "GraphElement"
    "GraphFrame"
    "GridMap"
    "GridMapEditorPlugin"
    "HFlowContainer"
    "HMACContext"
    "HeightMapShape3D"
    "HingeJoint3D"
    "IKModifier3D"
    "IPUnix"
    "ImageFormatLoader"
    "ImageFormatLoaderExtension"
    "ImageTexture3D"
    "ImageTextureLayered"
    "ImmediateMesh"
    "ImporterMesh"
    "ImporterMeshInstance3D"
    "InputEventFromWindow"
    "InputEventShortcut"
    "IntervalTweener"
    "IterateIK3D"
    "JNISingleton"
    "JacobianIK3D"
    "JavaClass"
    "JavaClassWrapper"
    "JavaObject"
    "JavaScriptBridge"
    "JavaScriptObject"
    "Joint3D"
    "JointLimitation3D"
    "JointLimitationCone3D"
    "KinematicCollision3D"
    "Label3D"
    "LabelSettings"
    "Light3D"
    "LightmapGI"
    "LightmapGIData"
    "LightmapProbe"
    "Lightmapper"
    "LightmapperRD"
    "LimitAngularVelocityModifier3D"
    "Logger"
    "LookAtModifier3D"
    "Marker2D"
    "Marker3D"
    "MenuBar"
    "MeshConvexDecompositionSettings"
    "MeshInstance3D"
    "MethodTweener"
    "MissingNode"
    "MissingResource"
    "MobileVRInterface"
    "ModifierBoneTarget3D"
    "MovieWriter"
    "MovieWriterMJPEG"
    "MovieWriterOGV"
    "MovieWriterPNGWAV"
    "MultiMeshInstance3D"
    "MultiplayerAPIExtension"
    "MultiplayerPeer"
    "MultiplayerPeerExtension"
    "MultiplayerSpawner"
    "MultiplayerSynchronizer"
    "NativeMenu"
    "NavigationAgent2D"
    "NavigationAgent3D"
    "NavigationLink2D"
    "NavigationLink3D"
    "NavigationMeshGenerator"
    "NavigationMeshSourceGeometryData2D"
    "NavigationMeshSourceGeometryData3D"
    "NavigationObstacle2D"
    "NavigationObstacle3D"
    "NavigationPathQueryParameters2D"
    "NavigationPathQueryParameters3D"
    "NavigationPathQueryResult2D"
    "NavigationPathQueryResult3D"
    "NavigationRegion2D"
    "NavigationRegion3D"
    "NavigationServer2D"
    "NavigationServer2DManager"
    "NavigationServer3D"
    "NavigationServer3DManager"
    "Node3D"
    "Node3DGizmo"
    "Noise"
    "NoiseTexture2D"
    "NoiseTexture3D"
    "ORMMaterial3D"
    "Occluder3D"
    "OccluderInstance3D"
    "OfflineMultiplayerPeer"
    "OggPacketSequence"
    "OggPacketSequencePlayback"
    "OmniLight3D"
    "OpenXRAPIExtension"
    "OpenXRAction"
    "OpenXRActionBindingModifier"
    "OpenXRActionMap"
    "OpenXRActionSet"
    "OpenXRAnalogThresholdModifier"
    "OpenXRAnchorTracker"
    "OpenXRAndroidThreadSettingsExtension"
    "OpenXRBindingModifier"
    "OpenXRBindingModifierEditor"
    "OpenXRCompositionLayer"
    "OpenXRCompositionLayerCylinder"
    "OpenXRCompositionLayerEquirect"
    "OpenXRCompositionLayerQuad"
    "OpenXRDpadBindingModifier"
    "OpenXRExtensionWrapper"
    "OpenXRExtensionWrapperExtension"
    "OpenXRFrameSynthesisExtension"
    "OpenXRFutureExtension"
    "OpenXRFutureResult"
    "OpenXRHand"
    "OpenXRHapticBase"
    "OpenXRHapticVibration"
    "OpenXRIPBinding"
    "OpenXRIPBindingModifier"
    "OpenXRInteractionProfile"
    "OpenXRInteractionProfileEditor"
    "OpenXRInteractionProfileEditorBase"
    "OpenXRInteractionProfileMetadata"
    "OpenXRInterface"
    "OpenXRMarkerTracker"
    "OpenXRPlaneTracker"
    "OpenXRRenderModel"
    "OpenXRRenderModelExtension"
    "OpenXRRenderModelManager"
    "OpenXRSpatialAnchorCapability"
    "OpenXRSpatialCapabilityConfigurationAnchor"
    "OpenXRSpatialCapabilityConfigurationAprilTag"
    "OpenXRSpatialCapabilityConfigurationAruco"
    "OpenXRSpatialCapabilityConfigurationBaseHeader"
    "OpenXRSpatialCapabilityConfigurationMicroQrCode"
    "OpenXRSpatialCapabilityConfigurationPlaneTracking"
    "OpenXRSpatialCapabilityConfigurationQrCode"
    "OpenXRSpatialComponentAnchorList"
    "OpenXRSpatialComponentBounded2DList"
    "OpenXRSpatialComponentBounded3DList"
    "OpenXRSpatialComponentData"
    "OpenXRSpatialComponentMarkerList"
    "OpenXRSpatialComponentMesh2DList"
    "OpenXRSpatialComponentMesh3DList"
    "OpenXRSpatialComponentParentList"
    "OpenXRSpatialComponentPersistenceList"
    "OpenXRSpatialComponentPlaneAlignmentList"
    "OpenXRSpatialComponentPlaneSemanticLabelList"
    "OpenXRSpatialComponentPolygon2DList"
    "OpenXRSpatialContextPersistenceConfig"
    "OpenXRSpatialEntityExtension"
    "OpenXRSpatialEntityTracker"
    "OpenXRSpatialMarkerTrackingCapability"
    "OpenXRSpatialPlaneTrackingCapability"
    "OpenXRSpatialQueryResultData"
    "OpenXRStructureBase"
    "OpenXRVisibilityMask"
    "OptimizedTranslation"
    "PacketPeerDTLS"
    "PacketPeerExtension"
    "PanoramaSkyMaterial"
    "Parallax2D"
    "ParticleProcessMaterial"
    "Path3D"
    "PathFollow3D"
    "PhysicalBone2D"
    "PhysicalBone3D"
    "PhysicalBoneSimulator3D"
    "PhysicalSkyMaterial"
    "PhysicsBody3D"
    "PhysicsDirectBodyState2D"
    "PhysicsDirectBodyState2DExtension"
    "PhysicsDirectBodyState3D"
    "PhysicsDirectBodyState3DExtension"
    "PhysicsDirectSpaceState2D"
    "PhysicsDirectSpaceState2DExtension"
    "PhysicsDirectSpaceState3D"
    "PhysicsDirectSpaceState3DExtension"
    "PhysicsPointQueryParameters2D"
    "PhysicsPointQueryParameters3D"
    "PhysicsRayQueryParameters2D"
    "PhysicsRayQueryParameters3D"
    "PhysicsServer2D"
    "PhysicsServer2DExtension"
    "PhysicsServer2DManager"
    "PhysicsServer3D"
    "PhysicsServer3DExtension"
    "PhysicsServer3DManager"
    "PhysicsServer3DRenderingServerHandler"
    "PhysicsShapeQueryParameters2D"
    "PhysicsShapeQueryParameters3D"
    "PhysicsTestMotionParameters2D"
    "PhysicsTestMotionParameters3D"
    "PhysicsTestMotionResult2D"
    "PhysicsTestMotionResult3D"
    "PinJoint3D"
    "PlaceholderCubemap"
    "PlaceholderCubemapArray"
    "PlaceholderMaterial"
    "PlaceholderMesh"
    "PlaceholderTexture2D"
    "PlaceholderTexture2DArray"
    "PlaceholderTexture3D"
    "PlaceholderTextureLayered"
    "PointLight2D"
    "PolygonOccluder3D"
    "PortableCompressedTexture2D"
    "ProceduralSkyMaterial"
    "PropertyTweener"
    "QuadOccluder3D"
    "RDAttachmentFormat"
    "RDFramebufferPass"
    "RDPipelineColorBlendState"
    "RDPipelineColorBlendStateAttachment"
    "RDPipelineDepthStencilState"
    "RDPipelineMultisampleState"
    "RDPipelineRasterizationState"
    "RDPipelineSpecializationConstant"
    "RDSamplerState"
    "RDShaderFile"
    "RDShaderSPIRV"
    "RDShaderSource"
    "RDTextureFormat"
    "RDTextureView"
    "RDUniform"
    "RDVertexAttribute"
    "RayCast3D"
    "RefCounted"
    "RegEx"
    "RegExMatch"
    "RemoteTransform3D"
    "RenderData"
    "RenderDataExtension"
    "RenderDataRD"
    "RenderSceneBuffers"
    "RenderSceneBuffersConfiguration"
    "RenderSceneBuffersExtension"
    "RenderSceneBuffersRD"
    "RenderSceneData"
    "RenderSceneDataExtension"
    "RenderSceneDataRD"
    "RenderingDevice"
    "RenderingServer"
    "ResourceFormatImporterSaver"
    "ResourceImporterBMFont"
    "ResourceImporterBitMap"
    "ResourceImporterCSVTranslation"
    "ResourceImporterDynamicFont"
    "ResourceImporterImage"
    "ResourceImporterImageFont"
    "ResourceImporterLayeredTexture"
    "ResourceImporterMP3"
    "ResourceImporterOBJ"
    "ResourceImporterOggVorbis"
    "ResourceImporterSVG"
    "ResourceImporterScene"
    "ResourceImporterShaderFile"
    "ResourceImporterTexture"
    "ResourceImporterTextureAtlas"
    "ResourceImporterWAV"
    "ResourceUID"
    "RetargetModifier3D"
    "RibbonTrailMesh"
    "RigidBody3D"
    "RuntimeNodeSelect"
    "SceneCacheInterface"
    "SceneMultiplayer"
    "SceneRPCInterface"
    "SceneReplicationConfig"
    "SceneReplicationInterface"
    "ScriptBacktrace"
    "ScriptEditorBase"
    "ScriptExtension"
    "ScriptLanguage"
    "ScriptLanguageExtension"
    "SeparationRayShape2D"
    "SeparationRayShape3D"
    "ShaderGlobalsOverride"
    "ShaderInclude"
    "ShaderIncludeDB"
    "Shape3D"
    "ShapeCast2D"
    "ShapeCast3D"
    "Shortcut"
    "Skeleton3D"
    "SkeletonIK3D"
    "SkeletonModification2D"
    "SkeletonModification2DCCDIK"
    "SkeletonModification2DFABRIK"
    "SkeletonModification2DJiggle"
    "SkeletonModification2DLookAt"
    "SkeletonModification2DPhysicalBones"
    "SkeletonModification2DStackHolder"
    "SkeletonModification2DTwoBoneIK"
    "SkeletonModificationStack2D"
    "SkeletonModifier3D"
    "SkeletonProfile"
    "SkeletonProfileHumanoid"
    "SliderJoint3D"
    "SocketServer"
    "SoftBody3D"
    "SphereOccluder3D"
    "SphereShape3D"
    "SplineIK3D"
    "SpotLight3D"
    "SpringArm3D"
    "SpringBoneCollision3D"
    "SpringBoneCollisionCapsule3D"
    "SpringBoneCollisionPlane3D"
    "SpringBoneCollisionSphere3D"
    "SpringBoneSimulator3D"
    "Sprite2D"
    "StandardMaterial3D"
    "StaticBody3D"
    "StatusIndicator"
    "StreamPeerExtension"
    "StreamPeerGZIP"
    "StreamPeerSocket"
    "StreamPeerTLS"
    "StreamPeerUDS"
    "SubViewport"
    "SubViewportContainer"
    "SubtweenTweener"
    "SyntaxHighlighter"
    "SystemFont"
    "TCPServer"
    "TLSOptions"
    "TabBar"
    "TextLine"
    "TextMesh"
    "TextParagraph"
    "TextServer"
    "TextServerAdvanced"
    "TextServerDummy"
    "TextServerExtension"
    "TextServerManager"
    "Texture2D"
    "Texture2DArray"
    "Texture2DArrayRD"
    "Texture2DRD"
    "Texture3DRD"
    "TextureCubemapArrayRD"
    "TextureCubemapRD"
    "TextureLayeredRD"
    "TextureProgressBar"
    "ThemeContext"
    "ThemeDB"
    "TileData"
    "TileMapLayer"
    "TileMapPattern"
    "TileSetAtlasSource"
    "TileSetScenesCollectionSource"
    "TileSetSource"
    "Time"
    "TorusMesh"
    "TranslationDomain"
    "TubeTrailMesh"
    "Tweener"
    "TwoBoneIK3D"
    "UDPServer"
    "UDSServer"
    "UPNP"
    "UPNPDevice"
    "UniformSetCacheRD"
    "VFlowContainer"
    "VehicleBody3D"
    "VehicleWheel3D"
    "VideoStreamPlayback"
    "VideoStreamPlayer"
    "VideoStreamTheora"
    "ViewPanner"
    "VisibleOnScreenEnabler2D"
    "VisibleOnScreenEnabler3D"
    "VisibleOnScreenNotifier2D"
    "VisibleOnScreenNotifier3D"
    "VisualInstance3D"
    "VisualShaderNodeBillboard"
    "VisualShaderNodeBooleanParameter"
    "VisualShaderNodeClamp"
    "VisualShaderNodeColorParameter"
    "VisualShaderNodeComment"
    "VisualShaderNodeConstant"
    "VisualShaderNodeCubemap"
    "VisualShaderNodeCubemapParameter"
    "VisualShaderNodeCurveTexture"
    "VisualShaderNodeCurveXYZTexture"
    "VisualShaderNodeDerivativeFunc"
    "VisualShaderNodeDistanceFade"
    "VisualShaderNodeFloatConstant"
    "VisualShaderNodeFloatFunc"
    "VisualShaderNodeFloatOp"
    "VisualShaderNodeFloatParameter"
    "VisualShaderNodeFrame"
    "VisualShaderNodeIntConstant"
    "VisualShaderNodeIntFunc"
    "VisualShaderNodeIntOp"
    "VisualShaderNodeIntParameter"
    "VisualShaderNodeLinearSceneDepth"
    "VisualShaderNodeMix"
    "VisualShaderNodeMultiplyAdd"
    "VisualShaderNodeParameter"
    "VisualShaderNodeParameterRef"
    "VisualShaderNodeParticleAccelerator"
    "VisualShaderNodeParticleBoxEmitter"
    "VisualShaderNodeParticleConeVelocity"
    "VisualShaderNodeParticleEmit"
    "VisualShaderNodeParticleEmitter"
    "VisualShaderNodeParticleMeshEmitter"
    "VisualShaderNodeParticleMultiplyByAxisAngle"
    "VisualShaderNodeParticleOutput"
    "VisualShaderNodeParticleRandomness"
    "VisualShaderNodeParticleRingEmitter"
    "VisualShaderNodeParticleSphereEmitter"
    "VisualShaderNodeProximityFade"
    "VisualShaderNodeRandomRange"
    "VisualShaderNodeRemap"
    "VisualShaderNodeReroute"
    "VisualShaderNodeResizableBase"
    "VisualShaderNodeRotationByAxis"
    "VisualShaderNodeSDFRaymarch"
    "VisualShaderNodeSDFToScreenUV"
    "VisualShaderNodeSample3D"
    "VisualShaderNodeScreenNormalWorldSpace"
    "VisualShaderNodeScreenUVToSDF"
    "VisualShaderNodeSmoothStep"
    "VisualShaderNodeStep"
    "VisualShaderNodeTexture2DArray"
    "VisualShaderNodeTexture2DArrayParameter"
    "VisualShaderNodeTexture2DParameter"
    "VisualShaderNodeTexture3D"
    "VisualShaderNodeTexture3DParameter"
    "VisualShaderNodeTextureParameter"
    "VisualShaderNodeTextureParameterTriplanar"
    "VisualShaderNodeTextureSDF"
    "VisualShaderNodeTextureSDFNormal"
    "VisualShaderNodeTransformOp"
    "VisualShaderNodeTransformParameter"
    "VisualShaderNodeUIntConstant"
    "VisualShaderNodeUIntFunc"
    "VisualShaderNodeUIntOp"
    "VisualShaderNodeUIntParameter"
    "VisualShaderNodeUVFunc"
    "VisualShaderNodeUVPolarCoord"
    "VisualShaderNodeVarying"
    "VisualShaderNodeVaryingGetter"
    "VisualShaderNodeVaryingSetter"
    "VisualShaderNodeVec2Constant"
    "VisualShaderNodeVec2Parameter"
    "VisualShaderNodeVec3Parameter"
    "VisualShaderNodeVec4Constant"
    "VisualShaderNodeVec4Parameter"
    "VisualShaderNodeVectorBase"
    "VisualShaderNodeWorldPositionFromDepth"
    "VoxelGI"
    "VoxelGIData"
    "WebRTCDataChannel"
    "WebRTCDataChannelExtension"
    "WebRTCMultiplayerPeer"
    "WebRTCPeerConnection"
    "WebRTCPeerConnectionExtension"
    "WebSocketMultiplayerPeer"
    "WebSocketPeer"
    "WebXRInterface"
    "Window"
    "WorkerThreadPool"
    "World3D"
    "WorldBoundaryShape2D"
    "WorldBoundaryShape3D"
    "XRAnchor3D"
    "XRBodyModifier3D"
    "XRBodyTracker"
    "XRCamera3D"
    "XRController3D"
    "XRControllerTracker"
    "XRFaceModifier3D"
    "XRFaceTracker"
    "XRHandModifier3D"
    "XRHandTracker"
    "XRInterface"
    "XRInterfaceExtension"
    "XRNode3D"
    "XROrigin3D"
    "XRPose"
    "XRPositionalTracker"
    "XRServer"
    "XRTracker"
    "XRVRS"
    "ZIPPacker"
    "ZIPReader"))

(provide 'gdscript-keywords)
;;; gdscript-keywords.el ends here
